
 /**
  * Include files
  */

#include <stdio.h>
#include <stdlib.h>

#include "ti/transport/ndk/nimu/example/ftpApp/modules.h"

#ifdef FTP_SERVER_MODULE_INCLUDED

#include <ti/csl/csl_error.h>
#include <ti/fs/fatfs/ff.h>
#include <ti/transport/ndk/nimu/example/ftpApp/ftpserver/ftpserver.h>

/**
 * Local functions
 */
static int32_t func_opts (io_handler_t *ioh);
static int32_t func_user (io_handler_t *ioh);
static int32_t func_pass (io_handler_t *ioh);
static int32_t func_stor (io_handler_t *ioh);
static int32_t func_dele (io_handler_t *ioh);
static int32_t func_rmd (io_handler_t *ioh);
static int32_t func_xrmd (io_handler_t *ioh);
static int32_t func_cwd (io_handler_t *ioh);
static int32_t func_cdup (io_handler_t *ioh);
static int32_t func_pwd (io_handler_t *ioh);
static int32_t func_xpwd (io_handler_t *ioh);
static int32_t func_retr (io_handler_t *ioh);
static int32_t func_list (io_handler_t *ioh);
static int32_t func_port (io_handler_t *ioh);
static int32_t func_nlst (io_handler_t *ioh);
static int32_t func_quit (io_handler_t *ioh);
static int fileInfoToStr(FILINFO *fno, char *buf);
static char *itoa(unsigned int value, char *string, int radix);

/**
 * Local variables
 */
ftp_cmd_handler_t cmd_handler[] =
{
    {"OPTS", 4, func_opts },
    {"USER", 4, func_user },
    {"PASS", 4, func_pass },
    {"STOR", 4, func_stor },
    {"DELE", 4, func_dele },
    {"RMD", 3, func_rmd },
    {"XRMD", 4, func_xrmd },
    {"CWD",  3, func_cwd },
    {"CDUP",  4, func_cdup },
    {"PWD",  3, func_pwd },
    {"XPWD", 4, func_xpwd },
    {"RETR", 4, func_retr },
    {"LIST", 4, func_list },
    {"PORT", 4, func_port },
    {"NLST", 4, func_nlst },
    {"LIST", 4, func_nlst },
    {"QUIT", 4, func_quit },
    {NULL, 0, NULL }
};

extern char ftpUser[NAME_AND_PASSWD_LEN];
extern char ftpPassword[NAME_AND_PASSWD_LEN];

/**
 * @brief  Return second argument from input string.
 */
static void argument_collector(char *_buffer, char  *_nameBuffer)
{
    /* TODO: Use malloc */
    char bf[1024];
    char sep[2] = " ";
    char *word;
    int32_t  wcount=0;

    strcpy(bf, _buffer);
    for (word = strtok(bf, sep);word;word = strtok(NULL, sep))
    {
        wcount++;
        if(wcount == 2)
        {
            strcpy(_nameBuffer, word);
        }
    }
}

/**
 * check_authentificated
 */
static boolean_t check_authentificated (io_handler_t *ioh)
{
    if (!ioh->authenticated)
    {
        sprintf(ioh->send_buffer,"530 Please log in with USER and PASS first. \r\n");
        send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
        return FALSE;
    }

    return TRUE;
}

/**
 * func_opts
 */
static int32_t func_opts (io_handler_t *ioh)
{
    int32_t bytes;

    argument_collector (ioh->receive_buffer, ioh->temp);

    sprintf(ioh->send_buffer,"530 Please log in with USER and PASS first.\r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}


/**
 * func_user
 */
static int32_t func_user (io_handler_t *ioh)
{
    int32_t bytes;

    argument_collector (ioh->receive_buffer, ioh->user);

    sprintf(ioh->send_buffer,"331 Password required \r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_pass
 */
static int32_t func_pass (io_handler_t *ioh)
{
    int32_t bytes;

    argument_collector (ioh->receive_buffer, ioh->password);
    if (strcmp (ioh->user, ftpUser) == 0 && strcmp (ioh->password, ftpPassword) == 0)
    {
        sprintf(ioh->send_buffer,"230 Public login successful \r\n");
        bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
        if (bytes < 0) return -1;
        ioh->authenticated = 1;
    }
    else
    {
        sprintf(ioh->send_buffer,"530 Invalid username or password \r\n");
        bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
        if (bytes < 0) return -1;
        ioh->authenticated = 0;
    }

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_dele
 */
static int32_t func_dele (io_handler_t *ioh)
{
    int32_t i;
    FRESULT errCode;
    FIL file;
    DIR dir;
    UINT br;
    FILINFO fno;
    char filename[256];
    int len;
    int32_t bytes;
    char path[64];

    if (!check_authentificated (ioh)) return -1;
    argument_collector(ioh->receive_buffer, path);

    len = strlen(ioh->current_working_dir_path);
    strncpy(filename, ioh->current_working_dir_path, 256);

    if(0 == len)
    {
        filename[0] = '\\';
        strncpy(&filename[1], path, 256-1);
    }
    else if(1 == len)
    {
        strncpy(&filename[1], path, 256-1);
    }
    else
    {
        if('\\' == filename[len-1])
        {
            strncpy(&filename[len], path, 256-len);
        }
        else
        {
            filename[len] = '\\';
            strncpy(&filename[len+1], path, 256-len-1);
        }
    }

    errCode = f_unlink(filename);

    if(FR_OK != errCode)
    {
        sprintf(ioh->send_buffer, "550 File not found\r\n");
    }
    else
    {
        sprintf(ioh->send_buffer, "250 File Delete Ok\r\n");
    }
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

static int32_t func_rmd (io_handler_t *ioh)
{
    int32_t i;
    FRESULT errCode;
    FIL file;
    DIR dir;
    UINT br;
    FILINFO fno;
    char filename[256];
    int len;
    int32_t bytes;
    char path[64];

    if (!check_authentificated (ioh)) return -1;
    argument_collector(ioh->receive_buffer, path);

    len = strlen(ioh->current_working_dir_path);
    strncpy(filename, ioh->current_working_dir_path, 256);

    if('\\' == path[0])
    {
        strncpy(&filename[0], path, 256);
    }
    else if(0 == len)
    {
        filename[0] = '\\';
        strncpy(&filename[1], path, 256-1);
    }
    else if(1 == len)
    {
        strncpy(&filename[1], path, 256-1);
    }
    else
    {
        if('\\' == filename[len-1])
        {
            strncpy(&filename[len], path, 256-len);
        }
        else
        {
            filename[len] = '\\';
            strncpy(&filename[len+1], path, 256-len-1);
        }
    }

    errCode = f_unlink(filename);

    if(FR_OK != errCode)
    {
        sprintf(ioh->send_buffer, "550 DIR not found\r\n");
    }
    else
    {
        sprintf(ioh->send_buffer, "250 DIR Delete Ok\r\n");
    }
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_xrmd
 */
static int32_t func_xrmd (io_handler_t *ioh)
{
    int32_t bytes;
    char path[64];

    if (!check_authentificated (ioh)) return -1;
    argument_collector(ioh->receive_buffer, path);

    if (0)
    {
        sprintf(ioh->send_buffer, "550 Directory not found\r\n");
    }
    else
    {
        sprintf(ioh->send_buffer, "200 Ok\r\n");
    }
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_stor
 */
static int32_t func_stor (io_handler_t *ioh)
{
    int32_t bytes;
    char filename[64];

    if (!check_authentificated (ioh)) return -1;

    argument_collector(ioh->receive_buffer, filename);

    //sprintf(ioh->send_buffer,"150 OK\r\n");
    sprintf(ioh->send_buffer, "550 Permission denied\r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
    return -1;

    if (ftp_filerout_write (ioh, filename) == 0)
    {
        sprintf(ioh->send_buffer, "226 File transfer completed\r\n");
    }
    else
    {
        sprintf(ioh->send_buffer, "450 Requested file action not taken\r\n");
    }

    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
    if (bytes > 0) return ERROR_SUCCESS;

    return -1;
}

/**
 * func_cwd
 */
static int32_t func_cwd (io_handler_t *ioh)
{
    int32_t bytes;
    char path[64];

    if (!check_authentificated (ioh)) return -1;
    argument_collector(ioh->receive_buffer, path);

    if((path[0] != 0) && (path[0] != '\\'))
    {
        int len = strlen(ioh->current_working_dir_path);

        if(0 == len)
        {
            ioh->current_working_dir_path[0] = '\\';
            snprintf(&(ioh->current_working_dir_path[1]), DATA_BUFFER_SIZE-len-1, "%s", path);
        }
        else if('\\' == ioh->current_working_dir_path[len-1])
        {
            snprintf(&(ioh->current_working_dir_path[len]), DATA_BUFFER_SIZE-len, "%s", path);
        }
        else
        {
            ioh->current_working_dir_path[len] = '\\';
            snprintf(&(ioh->current_working_dir_path[len+1]), DATA_BUFFER_SIZE-len-1, "%s", path);
        }
        sprintf(ioh->send_buffer, "250 CWD successful. %s is current directory\r\n", ioh->current_working_dir_path);
    }
    else if(path[0] == '\\')
    {
        snprintf(ioh->current_working_dir_path, DATA_BUFFER_SIZE, "%s", path);
        sprintf(ioh->send_buffer, "250 CWD successful. %s is current directory\r\n", ioh->current_working_dir_path);
    }
    else
    {
        sprintf(ioh->send_buffer, "550 CWD failed.\r\n");
    }
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

static int32_t func_cdup (io_handler_t *ioh)
{
    int32_t bytes;
    char path[64];

    if (!check_authentificated (ioh)) return -1;
    //argument_collector(ioh->receive_buffer, path);

        int len = strlen(ioh->current_working_dir_path);

        if(0 == len)
        {
            ioh->current_working_dir_path[0] = '\\';
            ioh->current_working_dir_path[1] = 0;
        }
        else if(1 == len)
        {
            //do nothing
        }
        else
        {
            int backidx = len-1;
            if('\\' == ioh->current_working_dir_path[len-1])
            {
                backidx--;
            }

            while(backidx >= 0)
            {
                if('\\' == ioh->current_working_dir_path[backidx])
                {
                    if(0 == backidx)
                    {
                        ioh->current_working_dir_path[1] = 0;
                    }
                    else
                    {
                        ioh->current_working_dir_path[backidx] = 0;
                    }
                    break;
                }
                backidx--;
            }
        }
        sprintf(ioh->send_buffer, "250 CWD successful. %s is current directory\r\n", ioh->current_working_dir_path);

    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_pwd
 */
static int32_t func_pwd (io_handler_t *ioh)
{
    int32_t bytes;
    if (!check_authentificated (ioh)) return -1;

    //sprintf(ioh->send_buffer, "257 \"/\" is current directory\r\n", ioh->current_working_dir_path);
    sprintf(ioh->send_buffer, "257 \"%s\" is current directory\r\n", ioh->current_working_dir_path);
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_xpwd
 */
static int32_t func_xpwd (io_handler_t *ioh)
{
    int32_t bytes;
    if (!check_authentificated (ioh)) return -1;

    sprintf(ioh->send_buffer, "257 \"%s\" is current directory\r\n", ioh->current_working_dir_path);
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_retr
 */
static int32_t func_retr (io_handler_t *ioh)
{
    int32_t bytes;
    char filename[64];
    if (!check_authentificated (ioh)) return -1;

    argument_collector(ioh->receive_buffer, filename);

    sprintf(ioh->send_buffer,"150 Transfering... \r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    if (ftp_filerout_read (ioh, filename) == CSL_SOK)
    {
        sprintf(ioh->send_buffer,"226 File transfer completed... \r\n");
    }
    else
    {
        sprintf(ioh->send_buffer,"450 Requested file action not taken\r\n");
    }
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    fdClose(ioh->data_socket);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_list
 */
static int32_t func_list (io_handler_t *ioh)
{
    int32_t bytes;
    char *buffer;
    DIR dp;
    FILINFO fno;
    char fileAttributeBuf[128];
    int fileAttributeLength = 0;
    int listLengthToSend = 0;

    if (!check_authentificated (ioh)) return -1;

    if ((buffer = malloc (16384)) != NULL)
    {
        sprintf(ioh->send_buffer,"150 Transfering... \r\n");
        bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
        //send(ioh->data_socket, buffer, strlen(buffer), 0);

        f_opendir(&dp, ioh->current_working_dir_path);

        while(1)
        {
            if(f_readdir (&dp, &fno) != FR_OK)
            {
                break;
            }

            if('\0' != *fno.fname)
            {
                fileAttributeLength = fileInfoToStr(&fno, fileAttributeBuf);
                if(listLengthToSend + fileAttributeLength > 16384)
                {
                    send(ioh->data_socket, buffer, listLengthToSend, 0);
                    memcpy(buffer, fileAttributeBuf, fileAttributeLength);
                    listLengthToSend = fileAttributeLength;
                }
                else
                {
                    memcpy(&buffer[listLengthToSend], fileAttributeBuf, fileAttributeLength);
                    listLengthToSend += fileAttributeLength;
                }
            }
            else
            {
                break;
            }
        }
        f_closedir(&dp);

        //sprintf(buffer,"02-23-05 09:24AM 2245 readme.ESn\r\n05-25-18 08:56AM 19041660 VC.txt\r\n06-25-18 08:56AM <DIR> data\r\n");
        if(listLengthToSend > 0)
        {
            send(ioh->data_socket, buffer, listLengthToSend, 0);
        }
        free (buffer);
    }

    sprintf(ioh->send_buffer,"226 File transfer completed... \r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    fdClose(ioh->data_socket);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_port
 */
static int32_t func_port (io_handler_t *ioh)
{
    int act_port[2];
    int act_ip[4], port_dec;
    char ip_decimal[40];
    struct sockaddr_in remoteaddr_act;
    int32_t bytes;

    if (!check_authentificated (ioh)) return -1;

    if (ioh->data_socket) fdClose(ioh->data_socket);
    ioh->data_socket = socket(AF_INET, SOCK_STREAMNC, IPPROTO_TCP);

    /* local variables */
    sscanf(ioh->receive_buffer, "PORT %d,%d,%d,%d,%d,%d", &act_ip[0], &act_ip[1], &act_ip[2], &act_ip[3], &act_port[0], &act_port[1]);
    sprintf(ip_decimal, "%d.%d.%d.%d", act_ip[0], act_ip[1], act_ip[2],act_ip[3]);
    port_dec = (act_port[0] * 256) + act_port[1];

    /* Prepare address for connect */
    memset( &remoteaddr_act,0, sizeof(struct sockaddr_in) );
    remoteaddr_act.sin_family = AF_INET;
    remoteaddr_act.sin_addr.s_addr = inet_addr(ip_decimal);
    remoteaddr_act.sin_port = NDK_htons(port_dec);

    if (connect (ioh->data_socket, (struct sockaddr *)&remoteaddr_act, (int32_t) sizeof(struct sockaddr_in)) == SOCKET_ERROR)
    {
        sprintf(ioh->send_buffer, "425 Something is wrong, can't start the active connection... \r\n");
        bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
        fdClose(ioh->data_socket);
    }
    else
    {
        sprintf(ioh->send_buffer, "200 Ok\r\n");
        bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
    }

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_nlst
 */
static int32_t func_nlst (io_handler_t *ioh)
{
    char *buffer;
    int32_t bytes;

    if (!check_authentificated (ioh)) return -1;
    if ((buffer = malloc (16384)) != NULL) /* Todo: User realloc () */
    {
        sprintf(ioh->send_buffer,"150 Transfering... \r\n");
        bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);
        send(ioh->data_socket, buffer, strlen(buffer), 0);
        free (buffer);
    }

    sprintf(ioh->send_buffer,"226 File transfer completed... \r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    fdClose(ioh->data_socket);

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

/**
 * func_quit
 */
static int32_t func_quit (io_handler_t *ioh)
{
    int32_t bytes;
    sprintf(ioh->send_buffer,"221 Connection closed by the FTP client\r\n");
    bytes = send(ioh->socket, ioh->send_buffer, strlen(ioh->send_buffer), 0);

    ioh->running = FALSE;

    if (bytes > 0) return ERROR_SUCCESS;
    return -1;
}

#endif

static int fileInfoToStr(FILINFO *fno, char *buf)
{
//    fsize:Indicates size of the file in unit of byte. Always zero for directories.
//    fdate:Indicates the date that the file was modified or the directory was created.
//    bit15:9
//    Year origin from 1980 (0..127)
//    bit8:5
//    Month (1..12)
//    bit4:0
//    Day (1..31)
//    ftime:Indicates the time that the file was modified or the directory was created.
//    bit15:11
//    Hour (0..23)
//    bit10:5
//    Minute (0..59)
//    bit4:0
//    Second / 2 (0..29)
//    fattrib:Indicates the file/directory attribute in combination of AM_DIR, AM_RDO, AM_HID, AM_SYS and AM_ARC.

    //output format
    //02-23-05 09:24AM 2245 readme.ESn\r\n
    //05-25-18 08:56AM 19041660 VC.txt\r\n
    //06-25-18 08:56AM <DIR> data\r\n

    int fileNameLength = 0;
    int fileSizeLength = 0;
    int idx = 0;

    if((fno != NULL) && (buf != NULL))
    {
        buf[0] = (char)((((fno->fdate) >> 5) & 0x0F) / 10) + '0';
        buf[1] = (char)((((fno->fdate) >> 5) & 0x0F) % 10) + '0';
        buf[2] = '-';
        buf[3] = (char)((((fno->fdate)) & 0x1F) / 10) + '0';
        buf[4] = (char)((((fno->fdate)) & 0x1F) % 10) + '0';
        buf[5] = '-';
        buf[6] = (char)(((((fno->fdate) >> 9) & 0x7F) - 20)/ 10) + '0';
        buf[7] = (char)(((((fno->fdate) >> 9) & 0x7F) - 20) % 10) + '0';
        buf[8] = ' ';
        buf[9] = (char)((((fno->ftime) >> 11) & 0x1F) / 10) + '0';
        buf[10] = (char)((((fno->ftime) >> 11) & 0x1F) % 10) + '0';
        buf[11] = ':';
        buf[12] = (char)((((fno->ftime) >> 5) & 0x3F) / 10) + '0';
        buf[13] = (char)((((fno->ftime) >> 5) & 0x3F) % 10) + '0';
        buf[14] = 'A';
        buf[15] = 'M';
        buf[16] = ' ';

        fileNameLength = strlen(fno->fname);
        if(fileNameLength > 80)
        {
            return 0;
        }
        if(!(fno->fattrib & AM_DIR))
        {
            //file
            itoa(fno->fsize, &buf[17], 10);
            fileSizeLength = strlen(&buf[17]);
            buf[17+fileSizeLength] = ' ';
            memcpy(&buf[18+fileSizeLength], fno->fname, fileNameLength);
            buf[18+fileSizeLength+fileNameLength] = '\r';
            buf[19+fileSizeLength+fileNameLength] = '\n';
            return 20+fileSizeLength+fileNameLength;
//            itoa(fno->fsize, &buf[15], 10);
//            fileSizeLength = strlen(&buf[15]);
//            buf[15+fileSizeLength] = ' ';
//            memcpy(&buf[16+fileSizeLength], fno->fname, fileNameLength);
//            buf[16+fileSizeLength+fileNameLength] = '\r';
//            buf[17+fileSizeLength+fileNameLength] = '\n';
//            return 18+fileSizeLength+fileNameLength;
        }
        else
        {
            //dir
            buf[17] = '<';
            buf[18] = 'D';
            buf[19] = 'I';
            buf[20] = 'R';
            buf[21] = '>';
            buf[22] = ' ';
            memcpy(&buf[23], fno->fname, fileNameLength);
            buf[23+fileNameLength] = '\r';
            buf[24+fileNameLength] = '\n';
            return 25+fileNameLength;
//            buf[15] = '<';
//            buf[16] = 'D';
//            buf[17] = 'I';
//            buf[18] = 'R';
//            buf[19] = '>';
//            buf[20] = ' ';
//            memcpy(&buf[21], fno->fname, fileNameLength);
//            buf[21+fileNameLength] = '\r';
//            buf[22+fileNameLength] = '\n';
//            return 23+fileNameLength;
        }
    }
    else
    {
        return 0;
    }
}

static char *itoa(unsigned int value, char *string, int radix)
{
    unsigned int     i, d;
    int     flag = 0;
    char    *ptr = string;

    /* This implementation only works for decimal numbers. */
    if (radix != 10)
    {
        *ptr = 0;
        return string;
    }

    if (!value)
    {
        *ptr++ = 0x30;
        *ptr = 0;
        return string;
    }

    for (i = 1000000000; i > 0; i /= 10)
    {
        d = value / i;

        if (d || flag)
        {
            *ptr++ = (char)(d + 0x30);
            value -= (d * i);
            flag = 1;
        }
    }

    /* Null terminate the string. */
    *ptr = 0;

    return string;

} /* NCL_Itoa */
