/* XDCtools Header files */
#include <xdc/std.h>
#include <xdc/cfg/global.h>
#include <xdc/runtime/System.h>
#include <stdio.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>

/* BIOS Header files */
#include <ti/sysbios/BIOS.h>
#ifdef MMCSD_EDMA_ENABLED
#include <ti/sysbios/family/arm/a15/Cache.h>
#include <ti/sdo/edma3/drv/edma3_drv.h>
#include <ti/sdo/edma3/rm/edma3_rm.h>
#include <ti/sdo/edma3/drv/sample/bios6_edma3_drv_sample.h>
#endif
#include <xdc/runtime/Error.h>

/* TI-RTOS Header files */
#include <ti/csl/cslr_device.h>
#include "MMCSD_log.h"
#include <ti/fs/fatfs/diskio.h>
#include <ti/fs/fatfs/FATFS.h>
#include <ti/drv/mmcsd/MMCSD.h>
#include <ti/drv/mmcsd/soc/MMCSD_soc.h>
#include <ti/drv/mmcsd/example/fatfs_console/src/fs_shell_app_utils.h>

#include <ti/board/board.h>

#include <ti/drv/gpio/GPIO.h>
#include <ti/drv/gpio/soc/GPIO_soc.h>

#define MMCSD_EDMA_ENABLED 1

#ifdef MMCSD_EDMA_ENABLED
#if defined(SOC_K2G)
/* MMCSD is connected to EDMA_1 for K2G */
#define MMCSD_EDMACC_NUM 1
#else
#define MMCSD_EDMACC_NUM 0
#endif
#endif

/**********************************************************************
 ************************** Macros ************************************
 **********************************************************************/


#define GPIO_PIN_MMC_SDCD_ACTIVE_STATE 0

/**********************************************************************
 ************************** Internal functions ************************
 **********************************************************************/


/**********************************************************************
 ************************** Global Variables **************************
 **********************************************************************/
volatile int dataReording = 0;
volatile int mountDisk = 0;

/* ON Board LED pins which are connected to GPIO pins. */
typedef enum GPIO_PIN {
    GPIO_PIN_MMC_SDCD      = 0U,
    GPIO_PIN_COUNT
}GPIO_PIN;

/* GPIO Driver board specific pin configuration structure */
GPIO_PinConfig gpioPinConfigs[] = {
    /* Output pin : AM335X_ICE V2_LD_PIN */
    GPIO_DEVICE_CONFIG(GPIO_MMC_SDCD_PORT_NUM, GPIO_MMC_SDCD_PIN_NUM) |
    GPIO_CFG_IN_INT_BOTH_EDGES | GPIO_CFG_INPUT,
};

/* GPIO Driver call back functions */
GPIO_CallbackFxn gpioCallbackFunctions[] = {
    NULL,
};

/* GPIO Driver configuration structure */
GPIO_v0_Config GPIO_v0_config = {
    gpioPinConfigs,
    gpioCallbackFunctions,
    sizeof(gpioPinConfigs) / sizeof(GPIO_PinConfig),
    sizeof(gpioCallbackFunctions) / sizeof(GPIO_CallbackFxn),
    0,
};


/* MMCSD function table for MMCSD implementation */
FATFS_DrvFxnTable FATFS_drvFxnTable = {
    MMCSD_close,
    MMCSD_control,
    MMCSD_init,
    MMCSD_open,
    MMCSD_write,
    MMCSD_read
};

/* FATFS configuration structure */
FATFS_HwAttrs FATFS_initCfg[_VOLUMES] =
{
    {
        0U
    },
    {
        1U
    },
    {
        2U
    },
    {
        3U
    }
};

/* FATFS objects */
FATFS_Object FATFS_objects[_VOLUMES];

/* FATFS configuration structure */
const FATFS_Config FATFS_config[_VOLUMES + 1] = {
    {
        &FATFS_drvFxnTable,
        &FATFS_objects[0],
        &FATFS_initCfg[0]
    },

    {
         &FATFS_drvFxnTable,
         &FATFS_objects[1],
         &FATFS_initCfg[1]
    },

    {
         &FATFS_drvFxnTable,
         &FATFS_objects[2],
         &FATFS_initCfg[2]
    },

    {NULL, NULL, NULL},

    {NULL, NULL, NULL}
};

/*
 *  ======== Callback function ========
 */
void AppGpioCallbackFxn(void)
{
   printf("AppGpioCallbackFxn\n");
}

 void board_initMMCSDGPIO(void)
{
    
#if defined(SOC_K2H) || defined(SOC_K2K) || defined(SOC_K2E) || defined(SOC_K2L) || defined(SOC_K2G) || defined(SOC_C6678) || defined(SOC_C6657) || defined(SOC_OMAPL137) || defined(SOC_OMAPL138)
    GPIO_v0_HwAttrs gpio_cfg;

    /* Get the default SPI init configurations */
    GPIO_socGetInitCfg(GPIO_MMC_SDCD_PORT_NUM, &gpio_cfg);

    /* Modify the default GPIO configurations if necessary */

    /* Set the default GPIO init configurations */
    GPIO_socSetInitCfg(GPIO_MMC_SDCD_PORT_NUM, &gpio_cfg);

#if defined(SOC_OMAPL137) || defined(SOC_OMAPL138)
    /* Setup GPIO interrupt configurations */
    GPIO_socSetBankInt(GPIO_MMC_SDCD_PORT_NUM, GPIO_MMC_SDCD_PIN_NUM, NULL);
#endif
#endif

}

/*
 *  ======== test function ========
 */
void mmcsd_init(void)
{
#ifdef MMCSD_EDMA_ENABLED
    EDMA3_DRV_Result edmaResult = 0;
    EDMA3_RM_Handle gEdmaHandle = NULL;
#if defined(SOC_OMAPL137) || defined(SOC_OMAPL138)
    MMCSD_v0_HwAttrs hwAttrsConfig;
#else
    MMCSD_v1_HwAttrs hwAttrsConfig;
#endif
#endif
    /* Perform board specific GPIO init  */
    board_initMMCSDGPIO();
#ifdef MMCSD_EDMA_ENABLED
    gEdmaHandle = (EDMA3_RM_Handle)edma3init(MMCSD_EDMACC_NUM, &edmaResult);

    if(MMCSD_socGetInitCfg(0,&hwAttrsConfig)!=0) {
       printf("\nUnable to obtain MMCSD config.Exiting. TEST FAILED.\r\n");
       return;
    }

    hwAttrsConfig.edmaHandle = gEdmaHandle;
    if(MMCSD_socSetInitCfg(0,&hwAttrsConfig)!=0) {
        printf("\nUnable to set config.Exiting. TEST FAILED.\r\n");
         return;
    }
#endif
    /* GPIO initialization */
    GPIO_init();

    /* Set the callback function */
    GPIO_setCallback(GPIO_PIN_MMC_SDCD, AppGpioCallbackFxn);

    /* Enable GPIO interrupt on the specific gpio pin */
    GPIO_enableInt(GPIO_PIN_MMC_SDCD);
}
#define DATA_BUF_ALIGN               (256)



#define MMCSD_TEST_MAX_BUFSIZE       (5*1024)

#define DATA_PATTERN_00     0
#define DATA_PATTERN_FF     1
#define SDMMC_DATA_PATTERN_AA 2
#define SDMMC_DATA_PATTERN_55 3
#define SDMMC_DATA_PATTERN_RAMP32 4
#define SDMMC_DATA_PATTERN_INC  8
#define SDMMC_STRESS_PATTERN_RANDOM_SMH 9
#define SDMMC_STRESS_PATTERN 10
#define FS_SHELL_APP_UTILS_PATH_BUF_SIZE (512)
#define MMCSD_TEST_NUM_SIZES (1U) /* 0.25 MB, 0.5 MB, 1 MB , 2MB , 5MB*/

uint32_t mmcsd_test_sizes[1]={(1024*1)};//{(1024*256),(1024*512),(1024*1024),(1024*1024*2),(1024*1024*3)};
uint8_t tx[MMCSD_TEST_MAX_BUFSIZE] __attribute__((aligned(DATA_BUF_ALIGN))) __attribute__((section (".benchmark_buffer")));
uint8_t rx[MMCSD_TEST_MAX_BUFSIZE] __attribute__((aligned(DATA_BUF_ALIGN))) __attribute__((section (".benchmark_buffer")));
/*****************************************************************************
A temporary data buffer used when manipulating file paths, or reading data
from the memory card.
******************************************************************************/
//static char MMCSD_FS_filepath[FS_SHELL_APP_UTILS_PATH_BUF_SIZE]="0:/temp_fatfs.txt";
static char MMCSD_FS_filepath[FS_SHELL_APP_UTILS_PATH_BUF_SIZE]="32.DAT";

/*****************************************************************************
A temporary data buffer used when manipulating file paths, or reading data
from the memory card.
******************************************************************************/
static FIL MMCSD_FS_WriteFileObj  __attribute__ ((aligned (SOC_CACHELINE_SIZE)));
/*****************************************************************************
A temporary data buffer used when manipulating file paths, or reading data
from the memory card.
******************************************************************************/
static FIL MMCSD_FS_ReadFileObj  __attribute__ ((aligned (SOC_CACHELINE_SIZE)));
uint32_t txRampBase=0;
int32_t fillMmcPageData(uint8_t *buf, int32_t length, uint8_t flag,uint32_t *rampBase)
{
    int32_t i;
    uint32_t data = 0x00;
    uint8_t incFlag = 0x00;

    switch(flag)
    {
        case DATA_PATTERN_00:
            data = 0;
            break;
        case DATA_PATTERN_FF:
            data = 0xFF;
            break;
        case SDMMC_DATA_PATTERN_AA:
            data = 0xAA;
            //MMCSD_log("\nSD Read/Write pattern 0xAA ");
            break;
        case SDMMC_DATA_PATTERN_55:
            data = 0x55;
            //MMCSD_log("\nSD Read/Write pattern 0x55 ");
            break;
        case SDMMC_DATA_PATTERN_INC:
             data = 0;
             incFlag = 0x01;
             //MMCSD_log("\nSD Read/Write incremental data");
             break;
        case SDMMC_DATA_PATTERN_RAMP32:
             //MMCSD_log("\nSD Read/Write 32 bit ramp ");
             break;
        default:
            return -1;
    }

    if(flag==SDMMC_DATA_PATTERN_RAMP32) {
         uint32_t *buf_32 = (uint32_t *)buf;
         for(i =0; i < length/sizeof(uint32_t); i++)  {
             *buf_32=*rampBase;
              buf_32++;
             *rampBase = *rampBase+1;
         }

    }
    else {
     for(i =0; i < length; i++)  {
        *(buf+i) = data;
         if(incFlag) {
            data++;
         }
      }
    }



    return 0;
}

/* test the mmcsd write read*/
int32_t HSMMCSDReadWriteTest_FATFS(void)
{
    int32_t j;
    FRESULT fresultRead = FR_NOT_READY;
    FRESULT fresultWrite = FR_NOT_READY;
    FRESULT fresultClose= FR_NOT_READY;
    FRESULT fresultDelete = FR_NOT_READY;
    FATFS_Handle fatfsHandle = NULL;

    uint32_t bytesWrite = 0;
    uint32_t bytesRead = 0;
    uint32_t flagWrite = FALSE;
    uint32_t flagRead = FALSE;
    int32_t fatfsTestRet=-1;
    uint32_t sizes;

    FATFS_Error fatfs_ret;
   // Task_sleep(15000);
    printf("\n\nPerforming FATFS read/write tests ..\n\n");
   /* MMCSD FATFS initialization */
   fatfs_ret=FATFS_init();
   if(fatfs_ret==FATFS_ERR) {
       printf("\n FATFS init failed!!");
     goto fatfs_test_exit;
   }

   /* Open FATFS instance */
   printf("\n FATFS open !!");
   fatfs_ret=FATFS_open(0U, NULL, &fatfsHandle);
   if(fatfs_ret != FATFS_OK) {
       printf("\n FATFS open failed!!");
     goto fatfs_test_exit;
   }
   printf("\n\fillMmcPageData FATFS start ..\n\n");

   /* Fill up the data pattern once */
   fillMmcPageData(&tx[0], (MMCSD_TEST_MAX_BUFSIZE), SDMMC_DATA_PATTERN_RAMP32,&txRampBase);
   printf("\n\fillMmcPageData FATFS end ..\n\n");

   for(sizes=0;sizes<MMCSD_TEST_NUM_SIZES;sizes++)
    {
  //     sizes = 0;
       /* Clear receive buffer */
       fillMmcPageData(&rx[0], mmcsd_test_sizes[sizes], DATA_PATTERN_00,NULL);

       fresultWrite = f_open(&MMCSD_FS_WriteFileObj, MMCSD_FS_filepath,FA_WRITE|FA_OPEN_ALWAYS);
       if(fresultWrite != FR_OK) {
         MMCSD_log("FATFS READ/WRITE: Fail to open file %s for write !!!!\n",MMCSD_FS_filepath);
         goto fatfs_test_exit;
       } else  {
         flagWrite = TRUE;
       }

       if(TRUE == flagWrite) {

         printf("\nFATFS READ/WRITE: Writing test pattern (%d KB) to file %s with \n",mmcsd_test_sizes[sizes]/1024,MMCSD_FS_filepath);
         fresultWrite = f_write(&MMCSD_FS_WriteFileObj, &tx[0],
                                         mmcsd_test_sizes[sizes], &bytesWrite);
         if(fresultWrite != FR_OK)  {
            printf("\n FATFS READ/WRITE: Fail to write into file with error code %d!!!!\n",fresultWrite);
            goto fatfs_test_exit;
         }

         if(bytesWrite!= mmcsd_test_sizes[sizes]) {
            printf("\n FATFS READ/WRITE: FAIL: Write operation wrote only %d of %d bytes",bytesWrite,mmcsd_test_sizes[sizes]);
            goto fatfs_test_exit;
         }
         /*
           * Close the Write file.
         */
#if 1
          if(TRUE == flagWrite){
            fresultClose = f_close(&MMCSD_FS_WriteFileObj);

            if(fresultClose != FR_OK) {
                printf("FATFS READ/WRITE: Fail to close write file with an error code %d!!!!\n",fresultWrite);
              goto fatfs_test_exit;
            }
          }
#endif

#if 0
          fresultWrite = f_open(&MMCSD_FS_WriteFileObj, MMCSD_FS_filepath,FA_WRITE);
          if(fresultWrite != FR_OK) {
            MMCSD_log("FATFS READ/WRITE: Fail to open file %s for write !!!!\n",MMCSD_FS_filepath);
            goto fatfs_test_exit;
          } else  {
            flagWrite = TRUE;
          }
#endif
       }

       fresultRead = f_open(&MMCSD_FS_ReadFileObj, MMCSD_FS_filepath, FA_READ);
       if(fresultRead != FR_OK)  {
          printf("FATFS READ/WRITE: Fail to open file %s for reading with return code %d!!!!\n",MMCSD_FS_filepath,fresultRead);
          goto fatfs_test_exit;
       }  else  {
          flagRead = TRUE;
       }
       if(TRUE == flagRead) {

        printf("FATFS READ/WRITE: Reading test pattern (%d KB) from file %s with \n",mmcsd_test_sizes[sizes]/1024,MMCSD_FS_filepath);
        fresultRead = f_read(&MMCSD_FS_ReadFileObj, &rx[0], mmcsd_test_sizes[sizes], &bytesRead);
        if(fresultRead != FR_OK) {
          printf("\n FATFS READ/WRITE: Fail to write into file!!!!\n");
          goto fatfs_test_exit;
        }

        if(bytesRead!= mmcsd_test_sizes[sizes]) {
          printf("\nFATFS READ/WRITE: FAIL: Read operation wrote only %d of %d bytes",bytesRead,mmcsd_test_sizes[sizes]);
          goto fatfs_test_exit;
        }
      }
       /* Close the read file */
        if(FR_OK == fresultClose){
            printf("read close the file\n");
          fresultClose = f_close(&MMCSD_FS_ReadFileObj);

          if(fresultClose != FR_OK) {
             printf("\nFATFS: Fail to close write file with an error code %d!!!!\n",fresultClose);
             goto fatfs_test_exit;
          }
        }

       /* Delete the read file */
        if(TRUE == fresultClose){
         printf("Delete the read file\n");
         fresultDelete = f_unlink(MMCSD_FS_filepath);

          if(fresultDelete != FR_OK) {
              printf("FATFS READ/WRITE: Fail to delect write file with an error code %d!!!!\n",fresultDelete);
              goto fatfs_test_exit;
          }
        }
        printf("Check the pattern\n");
        /* Check the pattern */
        for ( j = 0 ; j < (mmcsd_test_sizes[sizes]) ; j++ )
        {
            /* Compare the data read against the expected */
            if ( tx[j] != rx[j])
            {
                printf ("FATFS READ/WRITE: Data comparison failed at byte at %d ",j);
                printf(" ,rx[%d] = %x", rx[j],j);
                printf(" ,tx[%d] = %x", tx[j],j);
                goto fatfs_test_exit;
            }
         }
        printf ("FATFS PASS: Read/Write Success for this size (%d KB)  \r\n",mmcsd_test_sizes[sizes]/1024 );
        fatfsTestRet=0;
    }

fatfs_test_exit:
    if(fatfsHandle) {
        fatfs_ret=FATFS_close(fatfsHandle);
    if(fatfs_ret==FATFS_ERR) {
        printf("\n FATFS close failed!!");
        fatfsTestRet=-1;
        }
    }
    return fatfsTestRet;

}
void mmcsd_fatfs_console(UArg arg0, UArg arg1)
{

    HSMMCSDReadWriteTest_FATFS();
    /* MMCSD FATFS initialization */
    while(1){
        Task_sleep(1000);
        printf("sleep\n");
    }
}



