/*
 * netCmd.c
 *
 *  Created on: 2019412
 *      Author: rad-chenqz
 */

#include <stdio.h>

/* BIOS6 include */
#include <ti/sysbios/BIOS.h>

#include <ti/drv/uart/UART.h>
#include <ti/drv/uart/UART_stdio.h>
#include <ti/board/board.h>

#include <ti/ndk/inc/netmain.h>
#include <ti/ndk/inc/_stack.h>
#include <ti/ndk/inc/tools/console.h>
#include <ti/ndk/inc/tools/servers.h>
#include "ti/transport/ndk/nimu/example/client/src/client.h"

#include "soc_C6748.h"

#include "netCmd.h"

//bool g_bSonarWorking = TRUE;
extern volatile int dataReording;
extern volatile int mountDisk;

volatile CURRENTSTATUS g_current_status;
//volatile unsigned short g_auv_depth;
//volatile unsigned short g_auv_orientation;
//volatile unsigned short g_auv_pitch;
//volatile unsigned short g_auv_roll;
//volatile unsigned int g_auv_longitude;
//volatile unsigned int g_auv_latitude;
//volatile unsigned int g_auv_calander;
//volatile unsigned int g_auv_time;

void cmd_service()
{
    SOCKET   stcp = INVALID_SOCKET;
    SOCKET   sudp = INVALID_SOCKET;
    SOCKET   stcpactive = INVALID_SOCKET;
    SOCKET   stcpbusy;
    struct   sockaddr_in sin1;
    struct   timeval timeout;           /* Timeout struct for select */
    int      size,tmp;
    void  *hBuffer;
    char     pBuf[48];
    char     pReplyBuf[48];

    /* Allocate the file environment for this task */
    fdOpenSession( TaskSelf() );

    /* Set Port = 7, leaving IP address = Any */
    memset( &sin1, 0, sizeof(struct sockaddr_in) );
    sin1.sin_family = AF_INET;
    sin1.sin_port   = NDK_htons(7);

    /* Create our UDP echo socket */
    sudp = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if( sudp == INVALID_SOCKET )
        goto leave;

    /* Bind the same as TCP ... Port = 7, IPAddr = Any */
    if ( bind( sudp, (struct sockaddr *) &sin1, sizeof(sin1) ) < 0 )
        goto leave;

    /* Configure our timeout to be 15 seconds */
    timeout.tv_sec  = 15;
    timeout.tv_usec = 0;

    //DbgPrintf(DBG_INFO, "echosrv: EchoSrv Initialized\n");

    /* Run until task is destroyed by the system */
    for(;;)
    {
        NDK_fd_set ibits, obits, xbits;
        int    cnt;

        /* Clear the select flags */
        NDK_FD_ZERO(&ibits);
        NDK_FD_ZERO(&obits);
        NDK_FD_ZERO(&xbits);

        /* We examine the main TCP, UDP, and active TCP (if any) */
        NDK_FD_SET(sudp, &ibits);

        /* Check for new data on UDP socket */
        if( NDK_FD_ISSET(sudp, &ibits) )
        {
            tmp = sizeof( sin1 );
            cnt = (int)recvfrom( sudp, pBuf, 48, 0, (struct sockaddr *)&sin1, &tmp);

            if( cnt >= 0 )
            {
                if(processCmd(pBuf, cnt, pReplyBuf) == 1)
                {
                    sendto( sudp, pReplyBuf, 8, 0,(struct sockaddr *)&sin1, sizeof(sin1) );
                }
            }
        }
    }

leave:
    /* We only get here on an error - close the sockets */
    if( sudp != INVALID_SOCKET )
        fdClose( sudp );

    /* This task is killed by the system - here, we block */
    TaskDestroy( TaskSelf() );
}

//1 should reply, 0 no reply
static int processCmd(char *recvBuf, int recvNum, char *replyBuf)
{
    //const unsigned short * pCmdLength = (unsigned short *)(recvBuf + 2);
    const unsigned short * pCmdHead = (unsigned short *)(recvBuf + 4);

    *(unsigned short *)replyBuf = 0x7F7F; //head
    *(unsigned short *)(replyBuf + 2) = 0x02; //reply length

    if(0x7F7F != *(unsigned short *)recvBuf)
    {
        *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
        *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
        return 1;
    }

    switch (*pCmdHead)
    {
    case 0x0011: //start work
        {
            if(6 == recvNum)
            {
                dataReording = 1;
                mountDisk = 1;
                *(unsigned short *)(replyBuf + 4) = 0x0012;
                *(unsigned short *)(replyBuf + 6) = 0x0000;
            }
            else
            {
                *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
                *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
            }
        }
        return 1;

    case 0x0021:  //stop sonar
        {
            if(6 == recvNum)
            {
                dataReording = 0;
                mountDisk = 0;
                *(unsigned short *)(replyBuf + 4) = 0x0022;
                *(unsigned short *)(replyBuf + 6) = 0x0000;
            }
            else
            {
                *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
                *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
            }
        }
        return 1;

    case 0x0031:  //mru time sync
        {
            if(38 == recvNum)
            {
                memcpy(&g_current_status, recvBuf + 6, sizeof(g_current_status));
//                g_auv_depth = *(unsigned short *)(recvBuf + 6);
//                g_auv_orientation = *(unsigned short *)(recvBuf + 8);
//                g_auv_pitch = *(unsigned short *)(recvBuf + 10);
//                g_auv_roll = *(unsigned short *)(recvBuf + 12);
//                g_auv_longitude = *(unsigned int *)(recvBuf + 16);
//                g_auv_latitude = *(unsigned int *)(recvBuf + 20);
//                g_auv_calander = *(unsigned int *)(recvBuf + 24);
//                g_auv_time = *(unsigned int *)(recvBuf + 28);
                return 0;
            }
            else
            {
                *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
                *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
                return 1;
            }
        }
        break;

    case 0x0051:  //time sync
        {
            if(12 == recvNum)
            {
                unsigned int UserCalendar = 0;
                unsigned int UserTime = 0;

                //set rtc time
                //UserCalendar = 0x31121900;  date month year 00
                //UserTime = 0x10595900;  hour minite second
                if((recvBuf[7] <= 0x12) && (recvBuf[8] <= 0x31) && (recvBuf[9] <= 0x23) && (recvBuf[10] <= 0x59) && (recvBuf[11] <= 0x59))
                {
                    UserCalendar = ((unsigned int)recvBuf[6] << 8) | ((unsigned int)recvBuf[7] << 16) | ((unsigned int)recvBuf[8] << 24);
                    UserTime =  ((unsigned int)recvBuf[9] << 24) | ((unsigned int)recvBuf[10] << 16) | ((unsigned int)recvBuf[11] << 8);
                    RTCCalendarSet(SOC_RTC_0_REGS, UserCalendar);
                    RTCTimeSet(SOC_RTC_0_REGS, UserTime);
                    *(unsigned short *)(replyBuf + 4) = 0x0052;
                    *(unsigned short *)(replyBuf + 6) = 0x0000;
                    return 1;
                }
                else
                {
                    *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
                    *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
                    return 1;
                }
            }
            else
            {
                *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
                *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
                return 1;
            }
        }
        break;

    default:
        *(unsigned short *)(replyBuf + 4) = 0x00FF; //reply cmd
        *(unsigned short *)(replyBuf + 6) = 0x0001; //reply cmd result
        return 1;
    }
}

#if 0
static err_t cmd_tcp_client_recv(void *arg, struct tcp_pcb *pcb, struct pbuf *tcp_recv_pbuf, err_t err)
{
    //return 0;
#if 1
        struct pbuf *tcp_send_pbuf;
        struct name *name = (struct name *)arg;

        if (tcp_recv_pbuf != NULL)
        {
                /* շݵĴ */
                tcp_recved(pcb, tcp_recv_pbuf->len);
                if (tcp_recv_pbuf->len < 8)
                {
                    //pbuf_free(tcp_recv_pbuf);
                    return;
                }

                int recvNum = tcp_recv_pbuf->len;
                const unsigned short * pCmdHead = (unsigned short *)tcp_recv_pbuf->payload;
                const unsigned int* pCmdParam = (unsigned int*)((unsigned int)pCmdHead + 8);
                char txCmd[14];

                SonarData* pSonarData = (SonarData*)replyBuf;

                pSonarData->datalen = 13;

                *(unsigned short *)(replyBuf) = 0x50;
                *(unsigned short *)(replyBuf + 2) = 0x02;
                *(unsigned int *)(replyBuf + 4) = 13;
                replyBuf[12] = 0;

//                if (!name)
//                {
//                        pbuf_free(tcp_recv_pbuf);
//                        return ERR_ARG;
//                }


                switch (*pCmdHead)
                {
                case 0x0005:  //work parm
                    {
                        if (recvNum == 8 + 36 + 1)
                        {
                            memcpy((char *)&g_CurrentParam, (void *)pCmdParam, sizeof(g_CurrentParam));

                            g_bSendSampleNum = TRUE;
                            *(unsigned int *)(replyBuf + 8) = 0;

                            txCmd[0] =0x70;
                            txCmd[1] =0x61;
                            txCmd[2] =0x72;
                            txCmd[3] =0x61;
                            txCmd[4] = ((char)g_CurrentParam.txArray) & 0x0F;  //0x01;
                            txCmd[5] =0x00;
                            txCmd[6] =0x00;
                            txCmd[7] =0x0B;
                            txCmd[8] =0xB8;
                            txCmd[9] =0x00;
                            txCmd[10] =0x00;
                            txCmd[11] =0xE4;
                            txCmd[12] =0x29;
                            txCmd[13] =((char)g_CurrentParam.workMode) & 0x0F;  //0x01;

                            for(int i = 0; i < 14; i++)
                            {
                                XUartPs_SendByte(STDOUT_BASEADDRESS, txCmd[i]);
                            }
                        }
                        else
                        {
                            *(unsigned int *)(replyBuf + 8) = 2;
                        }
                        xil_printf("work parm \r\n");

                    }
                    break;
                case 0x0020: //start work
                    {
                        if (recvNum == 8 + 1)
                        {
                            g_bCleanFifo = TRUE;
                            g_bSendOpenSonar = TRUE;
                            g_bSonarWorking = TRUE;
                            *(unsigned int *)(replyBuf + 8) = 0;
                        }
                        else
                        {
                            *(unsigned int *)(replyBuf + 8) = 2;
                        }
                        xil_printf("start work \r\n");
                    }
                    break;
                case 0x0021:  //stop sonar
                    {
                        if (recvNum == 8 + 1)
                        {
                            g_bCleanFifo = TRUE;
                            g_bSendCloseSonar = TRUE;
                            g_bSonarWorking = FALSE;
                            *(unsigned int *)(replyBuf + 8) = 0;
                        }
                        else
                        {
                            *(unsigned int *)(replyBuf + 8) = 2;
                        }
                        xil_printf("stop sonar \r\n");
                    }
                    break;
                case 0x0042:  //system reset
                    {
                        if (recvNum == 8 + 1)
                        {
                            *(unsigned int *)(replyBuf + 8) = 0;
                            sleep(1);
                            exit(0);
                        }
                        else
                        {
                            *(unsigned int *)(replyBuf + 8) = 2;
                        }
                        xil_printf("system reset \r\n");
                    }
                    break;
                case 0x0088:  //time sync
                    {
                        if (recvNum == 8 + 16 + 1)
                        {
                            g_TxCount = pCmdParam[0];
                            g_TxTimeSecond = pCmdParam[1];
                            g_TxTimeMicroSecond = pCmdParam[2];
                            if(g_Sound_Speed10 != pCmdParam[3])
                            {
                                g_Sound_Speed10 = pCmdParam[3];

                                g_bSendSampleNum = TRUE;
                            }
                            *(unsigned int *)(replyBuf + 8) = 0;
                        }
                        else
                        {
                            *(unsigned int *)(replyBuf + 8) = 2;
                        }
                        xil_printf("time sync \r\n");
                    }
                    break;
                default:
                    *(unsigned int *)(replyBuf + 8) = 2;
                    xil_printf("unkown msg\r\n");
                    break;
                }

                /* յݿͽṹ */
                //tcp_send_pbuf = tcp_recv_pbuf;

                /* յתȥ */
                tcp_write(pcb, (void*)replyBuf, 13, 1);

                //pbuf_free(tcp_recv_pbuf);
        }
        else if (err == ERR_OK)
        {
            //server close tcp connection
            if(pcb != NULL )
            {
                tcp_close(pcb);
            }
            cmd_tcp_pcb_exsit = 0;
            return ERR_OK;
        }

        return ERR_OK;
#endif
}
#endif
